/***************************************************************************
                            QT Game -  description
                             -------------------
    begin                : December 2001
    copyright            : (C) 2001 by M. Heni
    email                :  martin1@heni-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ZAURALIGN_H
#define ZAURALIGN_H

// include files for QT
#include <qpe/qpeapplication.h>
#include <qmainwindow.h>
#include <qaction.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qwhatsthis.h>
#include <qstring.h>
#include <qpixmap.h>
#include <qmsgbox.h>
#include "kconfig.h"


#undef ZLOg
#ifdef ZLOG
#include <qtextstream.h>
#include <qfile.h>
extern QTextStream zlog;
#endif

// application specific includes
class GameView;
class GameDoc;



/**
  * This Class is the base class for your application. It sets up the main
  * window and providing a menubar, toolbar
  * and statusbar. For the main view, an instance of class ZauralignView is
  * created which creates your view.
  */
class ZauralignApp : public QMainWindow
{
  Q_OBJECT
  
  public:
    /** construtor */
    ZauralignApp(QPEApplication *app);
    /** destructor */
    ~ZauralignApp();
    /** initializes all QActions of the application */
    void initActions();
    /** initMenuBar creates the menu_bar and inserts the menuitems */
    void initMenuBar();
    /** setup the document*/
    void initDoc();
    /** setup the mainview*/
    void initView();
    /** setup the level menu */
    void initLevels();
    /** setup the Themes menu */
    void initThemes();
    /** find theme path */
    void findThemePath();

    /** overloaded for Message box on last window exit */
    bool queryExit();
  /** No descriptions */
  GameDoc * getDocument();

  public slots:

    /** generate a new document in the actual view */
    void slotFileNew();
    /** open a document */
    void slotFileEnd();
    /* Show the highscore dialog */
    void slotFileHighscore();
    /** exits the application */
    void slotFileQuit();
    /** Choose the level */
    void slotSettingsLevel(QAction *);
    /** Choose the theme */
    void slotSettingsTheme(QAction *);

    /** shows an about dlg*/
    void slotHelpAbout();

    /** Start the intro and enable new game*/
    void slotStartIntro();


    /** emitted after game over */
    void slotGameOver(int mode);
    QPEApplication *application() {return  mApp;}


  private:

    /** view is the main widget which represents your working area. The View
     * class should handle all events of the view widget.  It is kept empty so
     * you can create your view according to your application's needs by
     * changing the view class.
     */
    GameView *view;
    /** doc represents your actual document and is created only once. It keeps
     * information such as filename and does the serialization of your files.
     */
    GameDoc *mDoc;
  
    /** file_menu contains all items of the menubar entry "File" */
    QPopupMenu *fileMenu;
    /** settings contains all items of the menubar entry "Edit" */
    QPopupMenu *settingsMenu;
    /** level contains all items of the submenu entry "Levels" */
    QPopupMenu *levelMenu;
    /** themes contains all items of the submenu entry "themse" */
    QPopupMenu *themeMenu;
    /** view_menu contains all items of the menubar entry "Help" */
    QPopupMenu *helpMenu;
    /** actions for the application initialized in initActions() and used to en/disable them
      * according to your needs during the program */
    QAction *fileNew;
    QAction *fileEnd;
    QAction *fileHighscore;
    QAction *fileQuit;

    QActionGroup *settingsLevel;
    QActionGroup *settingsTheme;

    QAction *helpAboutApp;

    QPEApplication *mApp;
    KConfig *mConfig; 
    QString mTheme;
    QString mGrafixDir;
    QStringList mThemeList;

    #ifdef ZLOG
    QFile mFile;
    #endif

};
#endif 

