/***************************************************************************
                            QT Game -  description
                             -------------------
    begin                : December 2001
    copyright            : (C) 2001 by M. Heni
    email                :  martin1@heni-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ZAURALIGNVIEW_H
#define ZAURALIGNVIEW_H

// include files for QT
#include <qwidget.h>
#include <qpixmap.h>
#include <qcolor.h>
//#include <qrect.h>
//#include <qdatetime.h>
#include <qtimer.h>
#include <qcanvas.h>
#include <qdict.h>

// #include "quintalign.h"
#include "kconfig.h"

#define MAX_ANIM  6

// application specific includes
#include "zauraligndoc.h"

/**
 * This class provides an incomplete base for your application view. 
 */



class GameDoc;

class SpriteNotify : public QObject
{
  Q_OBJECT
  public:
    SpriteNotify() :QObject(0,0) {refCnt=0;}
    void emitSignal(QCanvasSprite *parent,int mode) {emit signalNotify(parent,mode);}
    int refCnt;
  signals:
    void signalNotify(QCanvasItem *,int);
};

class PSprite: public QCanvasSprite
{
  public:
    PSprite(QCanvasPixmapArray *p,QCanvas *c);
    ~PSprite();
    void advance(int stage);
    void prepareAnimation(int no,char a,char b,char dir,char speed);
    /**
     *  choose animation 0,1,2
     *  animation goes from anima[no]..animb[no]
     *  direction=1: cycle a->b
     *           =-1: cycle b->a
     *           =2  singleshot a->b
     *           =-2 singleshot b->a
     *           =0  no animation
     */
    void setAnimation(int no,double animmodify=1.0);
    int animation() {return animationNumber;}
    double animationModify() {return mAnimModify;}
    QObject *emitter(bool createnew=true);
    void deleteEmitter();
    void moveTo(int x,int y);
    void setSpeed(double s) {speed=s;}
    void setData(int i) {mData=i;}
    int rtti() const {return 1000;}
    int data() {return mData;}
    int width();
    int height();
    QCanvasPixmap* pixmap() const;


  private:
    SpriteNotify *mEmitter;
    char anima[MAX_ANIM];
    char animb[MAX_ANIM];
    char animdir[MAX_ANIM];
    char animspeed[MAX_ANIM];
    char animSpeedCnt;
    int animationNumber;
    int currentAnimDir;
    double targetX,targetY;
    double speed;
    int moveStatus;
    int mData;
    double mAnimModify;
    double mMoveSpeed;

};

class SpriteCache
{
  public:
    SpriteCache();
    ~SpriteCache();
    QCanvasPixmapArray *pixmapArray;
    double z;
    char anima[MAX_ANIM];
    char animb[MAX_ANIM];
    char animdir[MAX_ANIM];
    char animspeed[MAX_ANIM];
    double mMoveSpeed;
};




class GameView : public QCanvasView
{
  Q_OBJECT
  public:
    /** Constructor for the main view */
    GameView(QString grafixdir,QWidget *parent = 0, const char *name=0);
    /** Destructor for the main view */
    ~GameView();

    /**
     * Initialises the view (done in the construcor)
     */
    void initView(bool deleteall=true);
    void setGrafixDir(QString dir) {mGrafix=dir;}

    /** returns a pointer to the document connected to the view instance. Mind that this method requires a GameApp instance as a parent
     * widget to get to the window document pointer by calling the GameApp::getDocument() method.
     *
     * @see GameApp#getDocument
     */
    GameDoc *getDocument() const;

  void setBackground(QPixmap &background);

  QCanvasItem * drawGameOver(int sequence=0,bool movement=true,bool remove=false);
  void drawDecoration(bool remove=false);
  void drawIntro(bool remove=false);
  void drawScore(bool remove=false);
  void drawBoard(int size,int newsize,bool remove=false);
  QCanvasItem *drawPiece(int x,int y,char col,int no,int mode=0);
  QCanvasItem *getItem(QString s,unsigned int no=0);
  void deleteItem(QString s,unsigned int no=0);
  void deleteItem(QCanvasItem *item);
  void deleteAllItems();
  void preloadGraphics();
  void Debug();
  void changeTheme(QString path,bool deco);


  protected:
  void rotatePixmap(QPixmap *pixmap,int deg); 
  void scalePixmap(QPixmap *pixmap,double sx,double sy);
  void changeHSV(QPixmap *pixmap,int dh=0,int ds=0,int dv=0);
  void changeGrey(QPixmap *pixmap,int lighter=100);
  void resizeEvent(QResizeEvent *e);
  void contentsMousePressEvent ( QMouseEvent *m );
  void setAlphaBuffer(QPixmap * p,bool b);


  QPixmap *loadPixmap(QString filename,QString maskname=0);
	
  protected slots:
  /**
   * moving stone timer
   */
    void slotMoveTo(QCanvasItem *,int);
    void slotContinueIntro();



  private:
    QSize pieceSize;    // Size of a board piece
    QSize boardSize;    // Size of a board
    QSize numberSize;   // Size of a scorenumber
    QPoint boardOffset; // Top/Left of board
    QPoint scoreOffset; // Top/Left of score
    QPoint gameoverOffset; // Top/Left of game over animation
    QPoint gameoverDestination; // Top/Left of game over animation target
    QPoint newOffset;   // Top/Left of new pieces

    QCanvas *mCanvas;    // our drawing canvas
    QDict<QCanvasItem> mItemDict;  // Spritename lookup
    QDict<SpriteCache> mSpriteDict;  // Spriteproperty lookup
    QCanvasText *mIntroText;   // Intro text

    QString mGrafix;    // path to grafix dir
    KConfig *mConfig;   // configuration file

    QStringList mDecoration;  // List of decorations
    int mScoreDigits;   // how many score digits to show

    QColor mTextColor;  //Color of Canvas Text
	
};


#endif
